=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2011-01-19 00:00:44 +0000
+++ bzrlib/builtins.py	2011-01-20 00:55:45 +0000
@@ -4583,7 +4583,7 @@
     def run(self, verbose=False):
         from bzrlib import plugin
         self.outf.writelines(
-            plugin.describe_loaded_plugins(show_paths=verbose))
+            plugin.describe_plugins(show_paths=verbose))
 
 
 class cmd_testament(Command):

=== modified file 'bzrlib/crash.py'
--- bzrlib/crash.py	2011-01-19 00:14:00 +0000
+++ bzrlib/crash.py	2011-01-20 00:55:45 +0000
@@ -254,7 +254,7 @@
 
 
 def _format_plugin_list():
-    return ''.join(plugin.describe_loaded_plugins(show_paths=True))
+    return ''.join(plugin.describe_plugins(show_paths=True))
 
 
 def _format_module_list():

=== modified file 'bzrlib/plugin.py'
--- bzrlib/plugin.py	2011-01-19 00:14:00 +0000
+++ bzrlib/plugin.py	2011-01-20 00:55:45 +0000
@@ -82,37 +82,40 @@
     load_plugins([])
 
 
-def describe_loaded_plugins(show_paths=False):
-    """Generate text description of loaded plugins.
+def describe_plugins(show_paths=False):
+    """Generate text description of plugins.
+
+    Includes both those that have loaded, and those that failed to 
+    load.
 
     :param show_paths: If true,
     :returns: Iterator of text lines (including newlines.)
     """
     from inspect import getdoc
-    unreported_warnings = plugin_warnings.copy()
-    for name, plugin in sorted(plugins().items()):
-        version = plugin.__version__
-        if version == 'unknown':
-            version = ''
-        name_ver = '%s %s' % (name, version)
-        d = getdoc(plugin.module)
-        if d:
-            doc = d.split('\n')[0]
+    loaded_plugins = plugins()
+    all_names = sorted(list(set(
+        loaded_plugins.keys() + plugin_warnings.keys())))
+    for name in all_names:
+        if name in loaded_plugins:
+            plugin = load_plugins[name]
+            version = plugin.__version__
+            if version == 'unknown':
+                version = ''
+            yield '%s %s\n' % (name, version)
+            d = getdoc(plugin.module)
+            if d:
+                doc = d.split('\n')[0]
+            else:
+                doc = '(no description)'
+            yield ("   %s\n" % doc)
+            if show_paths:
+                yield ("   %s\n" % plugin.path())
+            del plugin
         else:
-            doc = '(no description)'
-        yield ("%s\n" % name_ver)
-        yield ("   %s\n" % doc)
-        if show_paths:
-            yield ("   %s\n" % plugin.path())
-        if name in unreported_warnings:
-            for line in unreported_warnings[name]:
+            yield "%s (failed to load)\n" % name
+        if name in plugin_warnings:
+            for line in plugin_warnings[name]:
                 yield "  ** " + line + '\n'
-            del unreported_warnings[name]
-        yield ("\n")
-    for name in sorted(unreported_warnings.keys()):
-        yield "%s (failed to load)\n" % name
-        for line in unreported_warnings[name]:
-            yield "  ** " + line + '\n'
         yield '\n'
 
 

=== modified file 'bzrlib/tests/test_plugins.py'
--- bzrlib/tests/test_plugins.py	2011-01-19 00:25:44 +0000
+++ bzrlib/tests/test_plugins.py	2011-01-20 00:55:45 +0000
@@ -38,7 +38,7 @@
 
 # TODO: Write a test for plugin decoration of commands.
 
-class TestPluginMixin(object):
+class BaseTestPlugins(tests.TestCaseInTempDir):
 
     def create_plugin(self, name, source=None, dir='.', file_name=None):
         if source is None:
@@ -99,7 +99,7 @@
         self.failUnless('bzrlib.plugins.%s' % name in sys.modules)
 
 
-class TestLoadingPlugins(tests.TestCaseInTempDir, TestPluginMixin):
+class TestLoadingPlugins(BaseTestPlugins):
 
     activeattributes = {}
 
@@ -303,7 +303,7 @@
             "it to 'bad_plugin_name_'\.")
 
 
-class TestPlugins(tests.TestCaseInTempDir, TestPluginMixin):
+class TestPlugins(BaseTestPlugins):
 
     def setup_plugin(self, source=""):
         # This test tests a new plugin appears in bzrlib.plugin.plugins().
@@ -779,7 +779,7 @@
                         ['+foo', '-bar'])
 
 
-class TestDisablePlugin(tests.TestCaseInTempDir, TestPluginMixin):
+class TestDisablePlugin(BaseTestPlugins):
 
     def setUp(self):
         super(TestDisablePlugin, self).setUp()
@@ -820,6 +820,7 @@
         self.assertLength(0, self.warnings)
 
 
+
 class TestLoadPluginAtSyntax(tests.TestCase):
 
     def _get_paths(self, paths):
@@ -843,7 +844,7 @@
                           os.pathsep.join(['batman@cave', '', 'robin@mobile']))
 
 
-class TestLoadPluginAt(tests.TestCaseInTempDir, TestPluginMixin):
+class TestLoadPluginAt(BaseTestPlugins):
 
     def setUp(self):
         super(TestLoadPluginAt, self).setUp()
@@ -959,3 +960,15 @@
         self.overrideEnv('BZR_PLUGINS_AT', 'test_foo@%s' % plugin_path)
         plugin.load_plugins(['standard'])
         self.assertTestFooLoadedFrom(plugin_path)
+
+
+class TestDescribePlugins(BaseTestPlugins):
+
+    def test_describe_plugins(self):
+        self.overrideAttr(plugin, 'plugin_warnings',
+            {'bad': ['Failed to load (just testing)']})
+        self.assertEquals("""\
+bad (failed to load)
+  ** Failed to load (just testing)
+
+""", ''.join(plugin.describe_plugins()))

